@@ -169,7 +169,7 @@ KIO::TransferJob* FlickrComm::sendRequest(ArgMap &args)
         
     // setup request specific arguments
     args["api_key"] = m_APIKey;
-    args["api_sig"] = generateMD5(args);
+    args.insert("api_sig", generateMD5(args));
     
     // formulate complete URL
     url += assembleArgs(args);
@@ -283,7 +283,7 @@ void FlickrComm::doWebAuthentication(const QString &frob)
     args["api_key"] = "c0134cf226b1187e3d79e4e1be03d1bf";
     args["perms"]   = "write";
     args["frob"]    = frob;
-    args["api_sig"] = generateMD5(args);
+    args.insert("api_sig", generateMD5(args));
 
     addr += "?" + assembleArgs(args);
     
