@@ -26,10 +26,9 @@
 #include <config.h>
 #endif
 
-#include <kmainwindow.h>
-
-
-class kflickrWidget;
+#include <kurl.h>
+#include <kapplication.h>
+#include <kparts/mainwindow.h>
 
 
 /**
@@ -37,23 +36,22 @@ class kflickrWidget;
  * our menu and main widgets are created and managed.
  * @short Application Main Window.
  * @author M. Asselstine <asselsm@gmail.com>
- * @version 0.5
+ * @version 0.6
  */
-class kflickr : public KMainWindow
+class kflickr : public KParts::MainWindow
 {
     Q_OBJECT
 public:
     kflickr();
     virtual ~kflickr();
-    
+        
+    void load(const KURL& url);
+
 private:
-    kflickrWidget *m_TheWidget;
-    
-public slots:
-    void setEditState(bool);
-    void setRemoveState(bool);
-    void setUploadState(bool);
-    void newBandwidthValue( const QString & );
+    void setupActions();
+            
+private:
+    KParts::ReadWritePart *m_part;
 };
 
 #endif // _KFLICKR_H_
