@@ -45,6 +45,7 @@
 #include <qcombobox.h>
 #include <kprogress.h>
 #include <qvaluelist.h>
+#include <qsizepolicy.h>
 #include <kiconloader.h>
 #include <kfiledialog.h>
 #include <qscrollview.h>
@@ -55,10 +56,11 @@
 #include <kapplication.h>
 
 kflickrWidget::kflickrWidget(QWidget* parent, const char* name, WFlags fl)
-        : QWidget(parent,name,fl),
-    m_currentUserIndex(-1),
-    m_progressDlg(0L),
-    m_uploadInProgress(false)
+    : kflickrWidgetUI(parent,name,fl)
+    , DCOPObject("kflickrWidgetInterface")
+    , m_currentUserIndex(-1)
+    , m_progressDlg(0L)
+    , m_uploadInProgress(false)
 {
     QValueList<int> widths;
     
@@ -68,44 +70,17 @@ kflickrWidget::kflickrWidget(QWidget* parent, const char* name, WFlags fl)
     // config settings group 
     config->setGroup("KFlickrWidget");
     
-    // main layout manager
-    QVBoxLayout* kflickrwidgetLayout = new QVBoxLayout(this, 11, 6); 
-    
-    // Users and Authentication widgets frame
-    QFrame* userFrame = new QFrame(this);
-    userFrame->setFrameShadow(QFrame::Raised);
-    userFrame->setFrameShape(QFrame::StyledPanel);
-    QGridLayout* userLayout = new QGridLayout(userFrame, 1, 1, 11, 6);
-    kflickrwidgetLayout->addWidget(userFrame);
-    
-    // The text labels
-    userLayout->addWidget(new QLabel(i18n("User:"), userFrame), 0, 0);
-    userLayout->addWidget(new QLabel(i18n("Photoset:"), userFrame), 1, 0);    
-
-    // The user and photoset combo boxes    
-    m_users = new QComboBox(userFrame);
-    m_users->setMinimumSize(QSize(300, 26));
+    // Stored user data
     m_users->insertStringList(config->readListEntry("users", QStringList()));
-    userLayout->addWidget(m_users, 0, 1);
-    
-    m_photosets = new QComboBox(userFrame);
-    m_photosets->setEditable(true);
-    m_photosets->setMinimumSize(QSize(300, 26));
-    userLayout->addWidget(m_photosets, 1, 1);    
-    
-    // Some Spacers
-    userLayout->addItem(new QSpacerItem(280, 30, QSizePolicy::Expanding, QSizePolicy::Minimum), 0, 2);    
-    userLayout->addItem(new QSpacerItem(280, 30, QSizePolicy::Expanding, QSizePolicy::Minimum), 1, 2);
-    
-    // New user button and another spacer
-    m_authNew = new QPushButton(i18n("Authenticate New User..."), userFrame);
-    userLayout->addWidget(m_authNew, 0, 3);
-    userLayout->addItem(new QSpacerItem(180, 30, QSizePolicy::Expanding, QSizePolicy::Minimum), 1, 3);
-   
-    // User authorization tokens
     m_userNSIDs= config->readListEntry( "user_nsids", QStringList() );
     m_userTokens = config->readListEntry( "user_tokens", QStringList() );
     
+    // Set GUI icons
+    m_addBtn->setGuiItem(KGuiItem(i18n("&Add"), "add"));
+    m_removeBtn->setGuiItem(KGuiItem(i18n("&Remove"), "remove"));
+    m_editBtn->setGuiItem(KGuiItem(i18n("&Edit"), "edit"));
+    m_uploadBtn->setGuiItem(KGuiItem(i18n("&Upload"), "up"));
+    
     // Photo list
     widths = config->readIntListEntry( "column_widths" );
     if( !widths.size() )
@@ -120,7 +95,9 @@ kflickrWidget::kflickrWidget(QWidget* parent, const char* name, WFlags fl)
     }
     
     // initial list setup
-    m_photolist = new PhotoListView( this, "m_photolist" );    
+    m_photolist = new PhotoListView( m_listHolder, "m_photolist" );
+    QVBoxLayout* listLayout = new QVBoxLayout(m_listHolder, 0, 6);
+    listLayout->addWidget(m_photolist);
     
     m_photolist->addColumn( i18n("Photo"), widths[0] );
     m_photolist->addColumn( i18n("Public"), widths[1] );
@@ -134,63 +111,39 @@ kflickrWidget::kflickrWidget(QWidget* parent, const char* name, WFlags fl)
     m_photolist->setAcceptDrops( TRUE );
     m_photolist->setDropVisualizer( TRUE );
     m_photolist->setSelectionMode( QListView::Extended );
-    
-    kflickrwidgetLayout->addWidget( m_photolist );
-    
-    // button frame
-    QFrame* btnFrame = new QFrame( this, "btnFrame");
-    btnFrame->setFrameShape(QFrame::StyledPanel);
-    btnFrame->setFrameShadow(QFrame::Raised);
-    QHBoxLayout* btnFrameLayout = new QHBoxLayout(btnFrame, 11, 6, "btnFrameLayout"); 
-
-    // add button
-    m_addBtn = new KPushButton(KGuiItem(i18n("&Add"), "add"), btnFrame);
-    btnFrameLayout->addWidget(m_addBtn);
-
-    // remove button
-    m_removeBtn = new KPushButton(KGuiItem(i18n("&Remove"), "remove"), btnFrame);
-    btnFrameLayout->addWidget(m_removeBtn);
-
-    // edit button
-    m_editBtn = new KPushButton(KGuiItem(i18n("&Edit"), "edit"), btnFrame);
-    btnFrameLayout->addWidget(m_editBtn);
-    
-    QSpacerItem* spacer = new QSpacerItem(320, 30, QSizePolicy::Expanding, QSizePolicy::Minimum);
-    btnFrameLayout->addItem(spacer);
-
-    // upload button
-    m_uploadBtn = new KPushButton(KGuiItem(i18n("&Upload"), "up"), btnFrame);
-    btnFrameLayout->addWidget(m_uploadBtn);
-    
-    kflickrwidgetLayout->addWidget(btnFrame);
-    
+        
     // widget signals and slots connections
-    connect(m_addBtn, SIGNAL(clicked()), this, SLOT(addPhotos()));
-    connect(m_removeBtn, SIGNAL(clicked()), this, SLOT(removePhotos()));
-    connect(m_uploadBtn, SIGNAL(clicked()), this, SLOT(uploadPhotos()));
-    connect(m_authNew, SIGNAL(clicked()), this, SLOT(authorizeNewUser()));
-    connect(m_editBtn, SIGNAL(clicked()), this, SLOT(onEditSelected()));
+    connect(m_addBtn, SIGNAL(clicked()), SLOT(addPhotos()));
+    connect(m_removeBtn, SIGNAL(clicked()), SLOT(removePhotos()));
+    connect(m_uploadBtn, SIGNAL(clicked()), SLOT(uploadPhotos()));
+    connect(m_authNew, SIGNAL(clicked()), SLOT(authorizeNewUser()));
+    connect(m_editBtn, SIGNAL(clicked()), SLOT(onEditSelected()));
     connect(this, SIGNAL(enableEdit(bool)), m_editBtn, SLOT(setEnabled(bool)));
     connect(this, SIGNAL(enableRemove(bool)), m_removeBtn, SLOT(setEnabled(bool)));
     connect(this, SIGNAL(enableUpload(bool)), m_uploadBtn, SLOT(setEnabled(bool)));
-    connect(m_photolist, SIGNAL(selectionChanged()), this, SLOT(updateActionStates()));
-    connect(m_users, SIGNAL(activated(const QString &)), this, SLOT(setActiveUser(const QString &)));
+    connect(m_photolist, SIGNAL(selectionChanged()), SLOT(updateActionStates()));
+    connect(m_users, SIGNAL(activated(const QString &)), SLOT(setActiveUser(const QString &)));
     connect(m_photolist, SIGNAL(dropped(QDropEvent*,QListViewItem*)),
-            this, SLOT(dropSlot(QDropEvent*,QListViewItem*)) );
+            SLOT(dropSlot(QDropEvent*,QListViewItem*)) );
+    connect(m_photolist, SIGNAL(itemAdded(QListViewItem*)), SLOT(updateCount(QListViewItem*)));
+    connect(m_photolist, SIGNAL(itemRemoved(QListViewItem*)), SLOT(updateCount(QListViewItem*)));
+
                 
     // Communications signals and slots connections
-    connect(&m_comm, SIGNAL(commError(const QString &)), this, SLOT(handleCommError(const QString &)));
-    connect(&m_comm, SIGNAL(returnedFrob(const QString &)), this, SLOT(doUserAuthentication(const QString &)));
+    connect(&m_comm, SIGNAL(commError(const QString &)), SLOT(handleCommError(const QString &)));
+    connect(&m_comm, SIGNAL(returnedFrob(const QString &)), SLOT(doUserAuthentication(const QString &)));
     connect(&m_comm, SIGNAL(returnedToken(const QString &, const QString &, const QString &)),
-            this, SLOT(addUser(const QString &, const QString &, const QString &)));
+            SLOT(addUser(const QString &, const QString &, const QString &)));
     connect(&m_comm, SIGNAL(returnedTags(const QStringList &)),
-            this, SLOT(setPersistentTags(const QStringList &)));
+            SLOT(setPersistentTags(const QStringList &)));
     connect(&m_comm, SIGNAL(returnedPhotosets(const QStringList &, const QString &)),
-            this, SLOT(setPhotosets(const QStringList &, const QString &)));
+            SLOT(setPhotosets(const QStringList &, const QString &)));
     connect(&m_comm, SIGNAL(returnedUploadStatus(const QString &)),
-            parent,SLOT(newBandwidthValue(const QString &)));
-    connect(&m_comm, SIGNAL(returnedUploadedOK(const QString &)), this, SLOT(photoUploadedOK(const QString &)));
-                
+            SLOT(newBandwidthValue(const QString &)));
+    connect(&m_comm, SIGNAL(returnedUploadedOK(const QString &)), SLOT(photoUploadedOK(const QString &)));
+    
+    connect(&m_timer, SIGNAL(timeout()), SLOT(refreshCount()));            
+    
     // Finally set the active user if one already authorized
     if( config->readEntry( "current_user", QString( "" ) ) != "" )
         setActiveUser( config->readEntry( "current_user" ) );
@@ -228,34 +181,92 @@ kflickrWidget::~kflickrWidget()
         delete m_progressDlg;
 }
 
+
+/*!
+    \fn kflickrWidget::addPhoto(const KURL &url)
+ */
+ void kflickrWidget::addPhoto(const KURL &url)
+ {
+    addPhoto(url, true);
+ }
+ 
+ 
+/*!
+    \fn kflickrWidget::addPhoto(const KURL &url, bool doPreview)
+ */
+PhotoListViewItem* kflickrWidget::addPhoto(const KURL &url, bool doPreview)
+{
+    // Create the photo in the list
+    PhotoListViewItem *photo = new PhotoListViewItem( m_photolist, url );
+    
+    // use the image filename as the default title
+    photo->setTitle( url.filename() );
+            
+    // get the previews
+    if( doPreview )
+    {
+        m_photolist->getPreview(url);
+    }
+    
+    // update the button states
+    updateActionStates();
+    
+    return photo;
+}
+
+
+/*!
+    \fn kflickrWidget::addPhotos()
+ */
 void kflickrWidget::addPhotos()
 {
-    KURL::List urls;
-        
     // allow user to select some jpeg files to add
-    urls =  KFileDialog::getOpenURLs( ":OpenPhoto", "*.jpg *.png *.gif|Photo Files",
+    KURL::List urls =  KFileDialog::getOpenURLs( ":OpenPhoto", "*.jpg *.png *.gif|Photo Files",
                                        this, i18n("Select Image(s)") );
-    
-    // ensure at least one selection
+                                       
+    addPhotos(urls);    
+}
+
+
+/*!
+    \fn kflickrWidget::addPhotos(const KURL::List &urls)
+ */
+void kflickrWidget::addPhotos(const KURL::List &urls)
+{
     if( !urls.empty() )
     {
-        KURL::List::iterator it;
+        KURL::List::const_iterator it;
         
         // add each selection to the photo list
         for( it = urls.begin(); it != urls.end(); ++it )
         {
-            PhotoListViewItem *photo = new PhotoListViewItem( m_photolist, (*it) );
-            
-            // use the image filename as the default title
-            photo->setTitle( (*it).filename() );
+            addPhoto(*it, false);
         }
     }
     
     // get the previews
     m_photolist->getPreviews(urls);
+}
+
+
+/*!
+    \fn kflickrWidget::addPhoto(const KURL &url,
+                                const QString &title,
+                                const QStringList &tags,
+                                const QString &desc)
+ */
+void kflickrWidget::addPhoto(const KURL &url,
+                             const QString &title,
+                             const QStringList &tags,
+                             const QString &desc)
+{
+    // Add the photo to the list
+    PhotoListViewItem *photo = addPhoto(url, true);
     
-    // update the button states
-    updateActionStates();
+    // Update the photo's attributes
+    if( title != QString::null )    photo->setTitle(title);
+    if( tags.count() > 0 )          photo->setTags(tags);
+    if( desc != QString::null )     photo->setDescription(desc);
 }
 
 
@@ -381,6 +392,31 @@ void kflickrWidget::doUserAuthentication( const QString &frob )
 }
 
 
+/*!
+    \fn kflickrWidget::newBandwidthValue(const QString &val)
+ */
+void kflickrWidget::newBandwidthValue(const QString &val)
+{
+    emit bandwidthValue(val);
+}
+
+
+/*!
+    \fn PhotoListView::updateCount(QListViewItem*)
+ */
+void kflickrWidget::updateCount(QListViewItem*)
+{
+    m_timer.start(500, true);
+}
+
+/*!
+    \fn PhotoListView::refreshCount()
+ */
+void kflickrWidget::refreshCount()
+{
+    emit photoCount(m_photolist->childCount());
+}
+
 /*!
     \fn kflickrWidget::addUser( const QString &name, const QString &token )
  */
